/*---------------------------------------------------------------------------
    Workfile:Recordset.cpp
    Description: . Implementation of Class CRecordset

    Copyright (C) 2002 KTS Computers
    All rights reserved.

    Author: Tom,Jais T Mathew
    Last Modtime:  6/8/2002 
    Last Revision: 
---------------------------------------------------------------------------*/

#include "stdafx.h"
#include "Recordset.h"
#include  <malloc.h> 
#include  <TCHAR.H>

enum { MINSQLBUFFERSIZE = 5048 };

EXTERN_C const GUID FAR CLSID_CADORecordset = { 0x00000535, 0, 0x10,{ 0x80,0,0,0xAA,0,0x6D,0x2E,0xA4}};
EXTERN_C const IID FAR IID_IADORecordset =  { 0x0000054F, 0, 0x10,{ 0x80,0,0,0xAA,0,0x6D,0x2E,0xA4}};

/*---------------------------------------------------------------------------
	Function Name: CRecordset::~CRecordset
	Return Value: 
	Parameters:
	Remarks:  Destructor for CRecordset
---------------------------------------------------------------------------*/
CRecordset::~CRecordset()
{
	HRESULT hr = 0;
	Close();
	if(m_pRecordset)
	{
		m_pRecordset->Release();
		m_pRecordset = NULL;
	}
}

/*---------------------------------------------------------------------------
	Function Name: CRecordset::Open 
	Return Value: 
	Parameters:LPCTSTR lpszSQL,
	Remarks:  Formats the sql query and calls recordset open method.
---------------------------------------------------------------------------*/
BOOL CRecordset::Open(LPCTSTR lpszSQL,...)
{
	ATLASSERT(!IsOpen()); // The recordset should not already be open.
	HRESULT hr = 0;
	BOOL bRetVal = FALSE;
	

	va_list args;
	va_start(args,lpszSQL);

	CComBSTR bstrSQL( FormatSQL(lpszSQL, args) );

	//create instance if not
	if(!m_pRecordset)
	{

		hr = CoCreateInstance(CLSID_CADORecordset,
							  NULL,
							  CLSCTX_INPROC_SERVER,
							  IID_IADORecordset, 
							  (LPVOID*)&m_pRecordset);

		//modified
		if(hr!=S_OK)
			return FALSE;		
		
	}
	if(	m_pRecordset && !IsOpen())
	{
		CComVariant varQuery(bstrSQL);
		CComVariant varEmpty;

		//added to get the record count
		m_pRecordset->CursorLocation  = adUseClient;
		//open query
		hr = m_pRecordset->Open(varQuery, _variant_t((_Connection*)m_rConn), adOpenForwardOnly, adLockReadOnly, adCmdUnknown);

		//modified
		if(hr!=S_OK)
			return FALSE;
	
		bRetVal = TRUE; 
	}
	
	return bRetVal;
}


/*---------------------------------------------------------------------------
	Function Name: IsOpen
	Return Value: TRUE if SUCCESS else FALSE
	Parameters:
	Remarks:  return TRUE if Recordset opened else FALSE
---------------------------------------------------------------------------*/

BOOL CRecordset::IsOpen()
{
	//  return the state of connection

	long lState = 0;
	 
	if(m_pRecordset)
	{
		HRESULT hr = m_pRecordset->get_State(&lState);
		//modified
		if(hr!=S_OK)
			return FALSE;
	}

	return lState;
	
}


/*---------------------------------------------------------------------------
	Function Name: CRecordset::Close 
	Return Value: TRUE if SUCCESS else FALSE
	Parameters:
	Remarks:  Close the recordset if it is Open
---------------------------------------------------------------------------*/
BOOL CRecordset::Close()
{
	if( IsOpen() )
	{
		HRESULT hr = m_pRecordset->Close();
		return TRUE;
	}
		
	return FALSE; 
}


/*---------------------------------------------------------------------------
	Function Name: CRecordset::IsBOF 
	Return Value: TRUE if BOF else FALSE
	Parameters:
	Remarks:  Checks Recordset::BOF
---------------------------------------------------------------------------*/
BOOL CRecordset::IsBOF()
{
	ATLASSERT(m_pRecordset);

	VARIANT_BOOL fValue(VARIANT_FALSE);
		
	if(!m_pRecordset)
	{
		ATLTRACE(TEXT("CRecordset::BOF() RecordSet COM Object not initialized\n"));
		return FALSE;
	}
	HRESULT hr = m_pRecordset->get_BOF(&fValue);
	
	return (fValue == VARIANT_TRUE);
}

/*---------------------------------------------------------------------------
	Function Name: CRecordset::IsEOF 
	Return Value: TRUE if EOF else FALSE
	Parameters:
	Remarks: Checks Recordset::EOF
---------------------------------------------------------------------------*/
BOOL CRecordset::IsEOF()
{
	ATLASSERT(m_pRecordset);

	VARIANT_BOOL fValue = VARIANT_FALSE;
		
	if(!m_pRecordset)
	{
		ATLTRACE(TEXT("CRecordset::EOF() RecordSet COM Object not initialized\n"));
		return FALSE;
	}
	HRESULT hr= m_pRecordset->get_EndOfFile(&fValue);
	
	return (fValue == VARIANT_TRUE);
}
/*---------------------------------------------------------------------------
	Function Name: CRecordset::MoveFirst 
	Return Value: TRUE if SUCCESS else FALSE
	Parameters:
	Remarks:  Moves to first record in the recordset
---------------------------------------------------------------------------*/
BOOL CRecordset::MoveFirst()
{
	ATLASSERT(m_pRecordset);

	if(!m_pRecordset)
	{
		ATLTRACE(TEXT("CRecordset::MoveFirst() RecordSet COM Object not initialized\n"));
		return FALSE;
	}
	HRESULT hr= m_pRecordset->MoveFirst();
	
	return (!FAILED(hr));
}

/*---------------------------------------------------------------------------
	Function Name: CRecordset::MoveNext 
	Return Value: TRUE if SUCCESS else FALSE
	Parameters:
	Remarks:  Moves to next record in the recordset
---------------------------------------------------------------------------*/
BOOL CRecordset::MoveNext()
{
	ATLASSERT(m_pRecordset);

	if(!m_pRecordset)
	{
		ATLTRACE(TEXT("CRecordset::MoveNext() RecordSet COM Object not initialized\n"));
		return FALSE;
	}
	HRESULT hr = m_pRecordset->MoveNext();
	
	return (!FAILED(hr));
}


/*---------------------------------------------------------------------------
	Function Name: CRecordset::GetFieldValue 
	Return Value: Returns column value for the given column index
	Parameters: lpszFieldName: The field whose value we want to retrieve.

	Remarks:  Returns column value for the given column index
---------------------------------------------------------------------------*/
BOOL CRecordset::GetFieldValue(LPCTSTR lpszFieldName, CComVariant& pvarValue)
{

	ATLASSERT(lpszFieldName != NULL);
	ATLASSERT(m_pRecordset);

	HRESULT hr = 0;
	
	if(!m_pRecordset)
	{
		ATLTRACE(TEXT("CRecordset::GetField() RecordSet COM Object not initialized\n"));
		return FALSE;
	}
	if (!lpszFieldName)
	{
		ATLTRACE(TEXT("lpszFieldName can not be NULL\n"));
		return FALSE;
	}

	CComPtr<Fields> pFields;
	hr = m_pRecordset->get_Fields(&pFields);
	
	CComPtr<Field> pField;
	hr = pFields->get_Item(CComVariant(lpszFieldName), &pField);
	
	hr = pField->get_Value(&pvarValue);
	
	return TRUE;
	
}

/*---------------------------------------------------------------------------
	Function Name: CRecordset::GetFieldValueString 
	Return Value: TRUE if seccessfully retrieved the string.
	Parameters: lpszFieldName: The name of the field to retrieve.
	            pbstrValue: Used to return the value of the field.
	Remarks:  NULL values are returned as empty strings.
---------------------------------------------------------------------------*/
BOOL CRecordset::GetFieldValueString(LPCTSTR lpszFieldName, CComBSTR& pbstrValue)
{
	BOOL bRetVal = FALSE;	
		
	CComVariant varValue;
	if (GetFieldValue(lpszFieldName, varValue))
	{
		HRESULT hr = 0;

		if (varValue.vt == VT_NULL)
		{
			pbstrValue = TEXT("");
		}
		else if (varValue.vt != VT_BSTR)
		{
			hr = varValue.ChangeType(VT_BSTR);
			pbstrValue = varValue.bstrVal;
		}
		else
		{
			pbstrValue = varValue.bstrVal;
		}
		bRetVal = TRUE;
	}


	return bRetVal;
}

/*---------------------------------------------------------------------------
	Function Name: CRecordset::FormatSQL
	Return Value: The formatted SQL.
	Parameters:
		lpszSQL: Unformatted SQl string
		va_list: Variable argument list pointer.
	Remarks:  Formats SQL string 
---------------------------------------------------------------------------*/
CComBSTR CRecordset::FormatSQL(LPCTSTR lpszSQL, va_list args)
{
	CComBSTR bstrValue;
	
	long lLength = lstrlen(lpszSQL)+1;
	lLength = (lLength > MINSQLBUFFERSIZE ? lLength : MINSQLBUFFERSIZE);
 	
	LPTSTR lpszTemp = new TCHAR[lLength];
	memset(lpszTemp,0,lLength);

	_vstprintf(lpszTemp, lpszSQL, args);

	bstrValue = lpszTemp;
	
	delete [] lpszTemp;
	
    return bstrValue;
}


long CRecordset::GetRecordCount()
{
	return m_pRecordset->GetRecordCount();
}
